/*
Atmel Prozessor ATTiny45  @ 8  MHz

Bremsshopper-Steuerung

Ein- und Ausgnge
PORT B 0 (5) = DC Kreis OK  LED 2
PORT B 1 (6) = Shopper aktiv LED 1
PORT B 2 (7) = Relais Kondensatoren laden
PORT B 3 (2) = Shopper IN
PORT B 4 (3) = ADC2 Uzk


3,50V  entspricht Uzk nenn 	= 100% Uzk  = 716 Einheiten der Analogmessung = 325V
3,85V Start Shoppen 		= 110% Uzk  = 787 Einheiten der Analogmessung = 357V
4,27V max Shoppen 			= 122% Uzk  = 887 Einheiten der Analogmessung = 398V
4,40V entspricht Uzk max 	= 125% Uzk  = 900 Einheiten der Analogmessung = 408V
2,63V entspricht Uzk min 	=  75% Uzk  = 537 Einheiten der Analogmessung = 243V




*/




#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

volatile unsigned char ausg=0;
volatile unsigned char soll=00;
volatile unsigned char shoppen=0;

// ******************************************************************* 
//                       PWM Zykluszeit = 6,656ms ==> Frequenz = 150Hz
SIGNAL(SIG_OVERFLOW0)           
{				

if (soll>100) soll =100;
if (ausg) ausg=0; else ausg=1;
if (ausg && shoppen) PORTB |= _BV(3); else PORTB &=~_BV(3);
if (ausg) (TCNT0=(255-soll)); else (TCNT0=(255-(100-soll)));
  


}


// ******************************************************************* 
//                       Initialisierung          
// ******************************************************************* 
void initial(void)
{    

TCNT0 = 255;        	//  TCNT0 Timer auf 126s einstellen 
TCCR0B = 3;        	// count with cpu clock/64
TIMSK = (1<<TOIE0); 	// enable TCNT0 overflow

sei();			// enable interrupts

ADMUX=0x02;		// PortB4 = ADC2 = Analog Eingang
ADCSRB=0x00;
ADCSRA=0x84;	// ADEN =1 / Prescaler = 16

DDRB = 0x0f;      // PortB  Ausgang, PortB3 =Analog Eingang



shoppen=0;


}



// ******************************************************************* 
//                       Hauptprogramm          
// ******************************************************************* 

void main(void)
{ 

// ******************************************************************* 
//              Variablen fr das Hauptprogramm       
// ******************************************************************* 
unsigned char i=0;
unsigned int sample=0;
unsigned int value=0;



initial();	// Initialisierung Ports, Timer

for (;;) {;             /* loop forever */ // Bearbeitungszeit =  s
  
    sample=0;  // setzt den Samplewert auf 0
    // Schleife zum Sammeln von 32 Mewerten
    for(i=0;i<30;i++)  
    {
      ADCSRA |= _BV(6);  // single conversion mode ein
      while(ADCSRA & (1<<6));  // warten bis Konvertierung abgeschlosen

      sample+=ADCW;  // Aufsummierung der Samplewerte
    }
    value=sample/30;  // Aritmethisches Mittel der Samplewerte (alle 8025sec @ 1Mhz)
	   
    // Auswertung der Medaten
    if ((value>537) && (value<900))  PORTB |= _BV(0); else PORTB &=~_BV(0); // Uzk OK
    if (value>787)  PORTB |= _BV(1); else PORTB &=~_BV(1); // shoppen ein
    if (value>537)  PORTB |= _BV(2); else PORTB &=~_BV(2); // Relais Kondensatorladung brcken


    if (value>787)  shoppen=1; else shoppen =0;
	if (value>787)  soll=value-787; else soll = 0; // 787 = 110% Uzk


    }    	//Ende Loop for ever        
} 		//Ende Hauptprogramm      












